package com.example.cristialexandru.mylamp;

import android.annotation.TargetApi;
import android.app.AlertDialog;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.bluetooth.BluetoothSocket;
import android.content.DialogInterface;
import android.content.Intent;
import android.graphics.Color;
import android.graphics.PorterDuff;
import android.graphics.drawable.StateListDrawable;
import android.os.Build;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.Button;
import android.widget.CompoundButton;
import android.widget.SeekBar;
import android.widget.Switch;
import android.widget.TextView;

import com.larswerkman.holocolorpicker.ColorPicker;
import com.larswerkman.holocolorpicker.OpacityBar;
import com.larswerkman.holocolorpicker.SVBar;
import com.larswerkman.holocolorpicker.SaturationBar;
import com.larswerkman.holocolorpicker.ValueBar;
import android.support.design.widget.Snackbar;

import java.io.IOException;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Random;
import java.util.Set;
import java.util.UUID;;

public class MainActivity extends AppCompatActivity {
    private ColorPicker picker = null;
    private SVBar svBar = null;
    private OpacityBar opacityBar = null;
    private Button button = null;
    private TextView text = null ;
    private SeekBar seekBar = null;
    private BluetoothAdapter BA = null;
    private Set<BluetoothDevice> pairedDevices = null;
    private Switch bulb_switch = null;
    private Switch chill_switch = null;
    private BluetoothSocket btSocket = null;
    private static final UUID MY_UUID =
            UUID.fromString("00001101-0000-1000-8000-00805F9B34FB");
    private OutputStream outStream = null;


    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        getMenuInflater().inflate(R.menu.main, menu);
        return true;
    }

    void connectBTH() {

        /**
         * Get paired devices and popup a menu to select which to connect to
         */
        pairedDevices = BA.getBondedDevices();

        ArrayList<String> devices_name = new ArrayList<String>();
        for(BluetoothDevice bt : pairedDevices) {
            devices_name.add(bt.getName());
        }
        final CharSequence[] cs = devices_name.toArray(new CharSequence[devices_name.size()]);

        AlertDialog.Builder builder = new AlertDialog.Builder(this);
        builder.setTitle("Select a device");
        builder.setItems(cs, new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface dialog, int which) {
                for(BluetoothDevice bt : pairedDevices) {
                    BluetoothDevice device = BA.getRemoteDevice(bt.getAddress());
                    if (device.getName().equals(cs[which])) {
                        try {
                            btSocket = device.createRfcommSocketToServiceRecord(MY_UUID);
                            BA.cancelDiscovery();
                            btSocket.connect();
                            outStream = btSocket.getOutputStream();
                        } catch (IOException e) {
                            Snackbar.make(findViewById(R.id.LinearLayout2),"Could not send message!", Snackbar.LENGTH_LONG).show();
                        }
                    }
                }
                Snackbar.make(findViewById(R.id.LinearLayout2),"Device connected", Snackbar.LENGTH_LONG).show();
            }
        });
        builder.show();

    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {

        /**
         * Select item from menu
         */
        switch (item.getItemId()) {
            case R.id.connect:
                connectBTH();
                return true;

            case R.id.version:
                Snackbar.make(findViewById(R.id.LinearLayout2),"Version 1.0, May 2016", Snackbar.LENGTH_LONG).show();
                return true;

            default:
                return super.onOptionsItemSelected(item);
        }
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {

        /**
         * Setup
         */
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
        seekBar = (SeekBar)findViewById(R.id.seekBar);
        bulb_switch = (Switch) findViewById(R.id.bulb_switch);
        chill_switch = (Switch) findViewById(R.id.chill_switch);

        BA = BluetoothAdapter.getDefaultAdapter();
        Intent turnOn = new Intent(BluetoothAdapter.ACTION_REQUEST_ENABLE);
        startActivityForResult(turnOn, 0);
        picker = (ColorPicker) findViewById(R.id.picker);


        /**
         * Add listener to bulb switch
         */
        bulb_switch.setOnCheckedChangeListener(new CompoundButton.OnCheckedChangeListener() {
            @Override
            public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
                String s = "white" + "\n";
                try {
                    if (outStream != null) {
                        outStream.write(s.getBytes());
                        Snackbar.make(findViewById(R.id.LinearLayout2),"aaaaa", Snackbar.LENGTH_LONG).show();
                    } else {
                        Snackbar.make(findViewById(R.id.LinearLayout2),"No device connected", Snackbar.LENGTH_LONG).show();
                    }
                } catch (IOException e) {
                    e.printStackTrace();
                    Snackbar.make(findViewById(R.id.LinearLayout2),"Could not send message!", Snackbar.LENGTH_LONG).show();
                }
            }
        });

        /**
         * Add listener to chill switch
         */
        chill_switch.setOnCheckedChangeListener(new CompoundButton.OnCheckedChangeListener() {
            @Override
            public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
                if (isChecked) {
                    int colour = picker.getColor();
                    int p = seekBar.getProgress();
                    int max = seekBar.getMax();
                    char red = (char) (Color.red(colour) * p / max);
                    char green = (char) (Color.green(colour) * p / max);
                    char blue = (char) (Color.blue(colour) * p / max);
                    Random randomGenerator = new Random();
                    int rnad = randomGenerator.nextInt(255);
                    String s = "chill" + String.format("%03d", (int) rnad) +
                            String.format("%03d", (int) red) +
                            String.format("%03d", (int) green) +
                            String.format("%03d", (int) blue) + "\n";
                    try {
                        if (outStream != null) {
                            outStream.write(s.getBytes());
                        } else {
                            Snackbar.make(findViewById(R.id.LinearLayout2),"No device connected", Snackbar.LENGTH_LONG).show();
                        }
                    } catch (IOException e) {
                        e.printStackTrace();
                        Snackbar.make(findViewById(R.id.LinearLayout2),"Could not send message, check your connection", Snackbar.LENGTH_LONG).show();
                    }
                    picker.setOldCenterColor(picker.getColor());
                }
            }});

        /**
         * Add listener to color picker
         */
        picker.setOnColorChangedListener(new ColorPicker.OnColorChangedListener() {

            @TargetApi(Build.VERSION_CODES.JELLY_BEAN)
            @Override
            public void onColorChanged(int color) {

                /**
                 * Get colour from listener and send command to uc
                 */
                int colour = picker.getColor();
                int p = seekBar.getProgress();
                int max = seekBar.getMax();
                char red = (char) (Color.red(colour) * p / max);
                char green = (char) (Color.green(colour) * p / max);
                char blue = (char) (Color.blue(colour) * p / max);
                seekBar.getProgressDrawable().setColorFilter(colour, PorterDuff.Mode.SRC_IN);

                bulb_switch.getTrackDrawable().setColorFilter(colour, PorterDuff.Mode.SRC_IN);
                chill_switch.getTrackDrawable().setColorFilter(colour, PorterDuff.Mode.SRC_IN);

                String s = "rgb" + String.format("%03d", (int) red) +
                        String.format("%03d", (int) green) +
                        String.format("%03d", (int) blue) + "\n";
                try {
                    if (outStream != null) {
                        outStream.write(s.getBytes());
                    } else {
                        Snackbar.make(findViewById(R.id.LinearLayout2),"No device connected", Snackbar.LENGTH_LONG).show();
                    }
                } catch (IOException e) {
                    //e.printStackTrace();
                    Snackbar.make(findViewById(R.id.LinearLayout2),"Could not send message!", Snackbar.LENGTH_LONG).show();
                }
                picker.setOldCenterColor(picker.getColor());
            }
        });

        /**
         * Add listener to intensity bar
         */
        seekBar.setOnSeekBarChangeListener(new SeekBar.OnSeekBarChangeListener() {
            @Override
            public void onProgressChanged(SeekBar seekBar, int progress, boolean fromUser) {
                int colour = picker.getColor();
                int p = seekBar.getProgress();
                int max = seekBar.getMax();
                char red = (char) (Color.red(colour) * p / max);
                char green = (char) (Color.green(colour) * p / max);
                char blue = (char) (Color.blue(colour) * p / max);

                String s = "rgb" + String.format("%03d", (int) red) +
                        String.format("%03d", (int) green) +
                        String.format("%03d", (int) blue) + "\n";
                try {
                    if (outStream != null) {
                        outStream.write(s.getBytes());
                    } else {
                        Snackbar.make(findViewById(R.id.LinearLayout2),"No device connected", Snackbar.LENGTH_LONG).show();
                    }
                } catch (IOException e) {
                    e.printStackTrace();
                    Snackbar.make(findViewById(R.id.LinearLayout2),"Could not send message!", Snackbar.LENGTH_LONG).show();
                }
                picker.setOldCenterColor(picker.getColor());
            }

            @Override
            public void onStartTrackingTouch(SeekBar seekBar) {

            }

            @Override
            public void onStopTrackingTouch(SeekBar seekBar) {

            }
        });


    }
}
