/*
	Proiect PM - Bluetooth Controlled Mood Lamp
	Cristi Alexandru Vasile
	334CA
*/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "usart.h"

/*
	Red LED		-> OC0A 	-> PB3
	Green LED	-> OC0B		-> PB4
	Blue LED 	-> OC2A 	-> PD7
	White LED	-> (N/A)	-> PD5
*/

/* Macros for setting the intensity of each color */
#define RED(x) OCR0A = (x);
#define GREEN(x) OCR0B = (x);
#define BLUE(x) OCR2A = (x);

/* Macros for the white LED */
#define WHITE_ON PORTD |= (1 << PD5);
#define WHITE_OFF PORTD &= ~(1 << PD5);
#define WHITE_TOGGLE PORTD ^= (1 << PD5);

/* States that the lamp can go through */
#define RGB_STATE 0
#define WHITE_STATE 1
#define CHILL_STATE 2
#define STROBE_STATE 3
volatile static int current_state;

/*
	Red LED		-> OC0A 	-> PB3
	Green LED	-> OC0B		-> PB4
	Blue LED 	-> OC2A 	-> PD7
	White LED	-> (N/A)	-> PD5
*/


/*
	Initializes the PWM according to the table
	Red LED		-> OC0A 	-> PB3
	Green LED	-> OC0B		-> PB4
	Blue LED 	-> OC2A 	-> PD7
*/

void PWM_Init()
{
    TCCR0A = (3 << WGM00) | (2 << COM0A0) | (2 << COM0B0);
    TCCR0B = (1 << CS00);

    TCCR2A = (3 << WGM20) | (2 << COM2A0);
    TCCR2B = (1 << CS20);

    OCR0A = 0x00;
    OCR0B = 0x00;
    OCR2A = 0x00;
}


/* Used for the chill state */
#define CHILL_LIMIT 10
#define CHILL_COUNT 2000

volatile char chill_seed = 0;
volatile float chill_r, chill_g, chill_b;
volatile float chill_rn, chill_gn, chill_bn;
volatile float chill_step_r, chill_step_g, chill_step_b;


/* Used to receive a command through Bluetooth */
#define BUFLEN 50
char buff[BUFLEN];
int buff_i = 0;
char value[4];
char c;

/*
	Suported commands:
	(x is a value between 000 and 255)
	red[x]				- sets the intensity of the red LED to x
	green[x]			- sets the intensity of the green LED to x
	blue[x]				- sets the intensity of the blue LED to x
	rgb[x][x][x]		- sets the intensities of all LEDs accordingly
	white 				- toggles the white led (ON/OFF)
	chill[x][x][x][x]	- starts the chill mode (the controller
						receives a seed for srand and a start colour;
						then it smoothly switches between random colours)
*/

void treat_commad() {
	char *rc;
		buff_i = 0;
		if (strncmp(buff,"red",3) == 0) {
			current_state = RGB_STATE;
			strncpy(value, buff + 3, 3);
			value[3] = '\0';
			c = (char) atoi(value);
			RED(c);
		} else if (strncmp(buff,"green",5) == 0) {
			current_state = RGB_STATE;
			strncpy(value, buff + 5, 3);
			value[3] = '\0';
			c = (char) atoi(value);
			GREEN(c);
		} else if (strncmp(buff,"blue",4) == 0) {
			current_state = RGB_STATE;
			strncpy(value, buff + 4, 3);
			value[3] = '\0';
			c = (char) atoi(value);
			BLUE(c);
		} else if (strncmp(buff,"rgb",3) == 0) {
			current_state = RGB_STATE;
			strncpy(value, buff + 3, 3);
			value[3] = '\0';
			c = (char) atoi(value);
			RED(c);
			strncpy(value, buff + 6, 3);
			value[3] = '\0';
			c = (char) atoi(value);
			GREEN(c);
			strncpy(value, buff + 9, 3);
			value[3] = '\0';
			c = (char) atoi(value);
			BLUE(c);
		} else if (strncmp(buff,"white",5) == 0) {
			current_state = WHITE_STATE;
				RED(0);
				GREEN(0);
				BLUE(0);
				WHITE_TOGGLE;
		} else if (strncmp(buff,"chill",5) == 0) {
			current_state = CHILL_STATE;
			strncpy(value, buff + 5, 3);
			value[3] = '\0';
			c = (char) atoi(value);
			chill_seed = c;
			strncpy(value, buff + 8, 3);
			value[3] = '\0';
			c = (char) atoi(value);
			chill_r = c;
			strncpy(value, buff + 11, 3);
			value[3] = '\0';
			c = (char) atoi(value);
			chill_g = c;
			strncpy(value, buff + 14, 3);
			value[3] = '\0';
			c = (char) atoi(value);
			chill_b = c;
		}
}

/* Bluetooth Receive interrupt */
ISR (USART0_RX_vect) {
	buff[buff_i++] = UDR0;

	/* If we have reached the end of the message */
	if(buff[buff_i - 1] == '\n') {
		treat_commad();
	}
}

int main() {

	/* Setup */
    DDRD |= (1 << PD7) | (1 << PD5);
    DDRB |= (1 << PB3) | (1 << PB4);

    WHITE_OFF;
    PWM_Init();
    sei();
    USART0_init();

    srand(0);

	while (1) {
		if (current_state == RGB_STATE) {
			WHITE_OFF;
		} else if (current_state == WHITE_STATE) {
			/* Do nothing */
		} else if (current_state == CHILL_STATE) {
			WHITE_OFF;
			srand(chill_seed * 31 * 2531);

			/* Calculate the next random colour */
			chill_rn = (rand() % 255);
			chill_gn = (rand() % 255);
			chill_bn = (rand() % 255);

			/* Calculate the discrete step to add on each cycle */
			chill_step_r = (float) (chill_rn - chill_r) / CHILL_COUNT;
			chill_step_g = (float) (chill_gn - chill_g) / CHILL_COUNT;
			chill_step_b = (float) (chill_bn - chill_b) / CHILL_COUNT;

			/* Iterate */
			while (current_state == CHILL_STATE) {
				RED((int)chill_r);
				GREEN((int)chill_g);
				BLUE((int)chill_b);

				/* If the distance between colours is smaller
					than the limit, change colour */
				if (sqrt(	(chill_rn - chill_r) * (chill_rn - chill_r) +
							(chill_gn - chill_g) * (chill_gn - chill_g) +
							(chill_bn - chill_b) * (chill_bn - chill_b)) < CHILL_LIMIT) {

					chill_rn = ((double) rand() / RAND_MAX) * 255;
					chill_gn = ((double) rand() / RAND_MAX) * 255;
					chill_bn = ((double) rand() / RAND_MAX) * 255;

					chill_step_r = (chill_rn - chill_r) / CHILL_COUNT;
					chill_step_g = (chill_gn - chill_g) / CHILL_COUNT;
					chill_step_b = (chill_bn - chill_b) / CHILL_COUNT;
				}

				/* Next step */
				chill_r += chill_step_r;
				chill_g += chill_step_g;
				chill_b += chill_step_b;

				RED(0);
				GREEN(0);
				BLUE(0);
			}
		}
	}
    return 0;
}
